#!/usr/bin/env python

"""
Common scheduling functionality.

Copyright (C) 2015, 2016 Paul Boddie <paul@boddie.org.uk>

This program is free software; you can redistribute it and/or modify it under
the terms of the GNU General Public License as published by the Free Software
Foundation; either version 3 of the License, or (at your option) any later
version.

This program is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
details.

You should have received a copy of the GNU General Public License along with
this program.  If not, see <http://www.gnu.org/licenses/>.
"""

from imiptools.handlers.scheduling.manifest import scheduling_functions

def apply_scheduling_functions(functions, handler):

    """
    Apply the given scheduling 'functions' in the current object of the given
    'handler'.
    """

    response = "ACCEPTED"

    for fn in functions:

        # NOTE: Should signal an error for incorrectly configured resources.

        if not fn:
            return "DECLINED"

        # Keep evaluating scheduling functions, stopping only if one
        # declines or gives a null response.

        else:
            result = fn(handler)

            # Return a negative result immediately.

            if not result or result == "DECLINED":
                return result

            # Modify the eventual response from acceptance if a countering
            # result is obtained.

            elif response == "ACCEPTED":
                response = result

    return response

# vim: tabstop=4 expandtab shiftwidth=4
